=== modified file 'libdrizzle-1.0/conn_local.h'
--- libdrizzle-1.0/conn_local.h	2011-11-07 15:04:16 +0000
+++ libdrizzle-1.0/conn_local.h	2011-12-19 16:28:27 +0000
@@ -1,4 +1,5 @@
-/*
+/* vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
  * Drizzle Client & Protocol Library
  *
  * Copyright (C) 2008 Eric Day (eday@oddments.org)
@@ -69,6 +70,11 @@
  */
 static inline bool drizzle_state_none(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return false;
+  }
+
   return con->state_current == 0;
 }
 

=== modified file 'libdrizzle/column.cc'
--- libdrizzle/column.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/column.cc	2011-12-19 16:28:27 +0000
@@ -355,6 +355,11 @@
 drizzle_column_st *drizzle_column_create(drizzle_result_st *result,
                                          drizzle_column_st *column)
 {
+  if (result == NULL)
+  {
+    return NULL;
+  }
+
   if (column == NULL)
   {
     column= new (std::nothrow) (drizzle_column_st);
@@ -420,6 +425,11 @@
 
 void drizzle_column_free(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   if (column->result->column_list == column)
     column->result->column_list= column->next;
 
@@ -437,83 +447,163 @@
 
 drizzle_result_st *drizzle_column_drizzle_result(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return NULL;
+  }
+
   return column->result;
 }
 
 const char *drizzle_column_catalog(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return NULL;
+  }
+
   return column->catalog;
 }
 
 const char *drizzle_column_db(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return NULL;
+  }
+
   return column->db;
 }
 
 const char *drizzle_column_table(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return NULL;
+  }
+
   return column->table;
 }
 
 const char *drizzle_column_orig_table(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return NULL;
+  }
+
   return column->orig_table;
 }
 
 const char *drizzle_column_name(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return NULL;
+  }
+
   return column->name;
 }
 
 const char *drizzle_column_orig_name(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return NULL;
+  }
+
   return column->orig_name;
 }
 
 drizzle_charset_t drizzle_column_charset(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return drizzle_charset_t();
+  }
+
   return column->charset;
 }
 
 uint32_t drizzle_column_size(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return 0;
+  }
+
   return column->size;
 }
 
 size_t drizzle_column_max_size(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return 0;
+  }
+
   return column->max_size;
 }
 
 void drizzle_column_set_max_size(drizzle_column_st *column, size_t size)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   column->max_size= size;
 }
 
 drizzle_column_type_t drizzle_column_type(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return drizzle_column_type_t();
+  }
+
   return column->type;
 }
 
 drizzle_column_type_drizzle_t
 drizzle_column_type_drizzle(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return drizzle_column_type_drizzle_t();
+  }
+
   return _column_type_drizzle_map_from[column->type];
 }
 
 drizzle_column_flags_t drizzle_column_flags(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return drizzle_column_flags_t();
+  }
+
   return drizzle_column_flags_t(column->flags);
 }
 
 uint8_t drizzle_column_decimals(drizzle_column_st *column)
 {
+  if (column == NULL)
+  {
+    return 0;
+  }
+
   return column->decimals;
 }
 
 const uint8_t *drizzle_column_default_value(drizzle_column_st *column,
                                             size_t *size)
 {
+  if (column == NULL)
+  {
+    return NULL;
+  }
+
   *size= column->default_value_size;
   return column->default_value;
 }
@@ -524,6 +614,11 @@
 
 drizzle_return_t drizzle_column_skip(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_return_t ret;
   if (drizzle_state_none(result->con))
   {
@@ -539,6 +634,11 @@
 
 drizzle_return_t drizzle_column_skip_all(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   for (uint16_t it= 1; it <= result->column_count; it++)
   {
     drizzle_return_t ret= drizzle_column_skip(result);
@@ -556,6 +656,17 @@
                                        drizzle_column_st *column,
                                        drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (result == NULL)
+  {
+    return NULL;
+  }
+
   if (drizzle_state_none(result->con))
   {
     result->column= column;
@@ -571,6 +682,11 @@
 
 drizzle_return_t drizzle_column_buffer(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_return_t ret;
 
   if (result->column_buffer == NULL)
@@ -605,6 +721,11 @@
 
 drizzle_column_st *drizzle_column_next(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return NULL;
+  }
+
   if (result->column_current == result->column_count)
   {
     return NULL;
@@ -616,6 +737,11 @@
 
 drizzle_column_st *drizzle_column_prev(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return NULL;
+  }
+
   if (result->column_current == 0)
   {
     return NULL;
@@ -627,6 +753,11 @@
 
 void drizzle_column_seek(drizzle_result_st *result, uint16_t column)
 {
+  if (result == NULL)
+  {
+    return;
+  }
+
   if (column <= result->column_count)
   {
     result->column_current= column;
@@ -636,6 +767,11 @@
 drizzle_column_st *drizzle_column_index(drizzle_result_st *result,
                                         uint16_t column)
 {
+  if (result == NULL)
+  {
+    return NULL;
+  }
+
   if (column >= result->column_count)
   {
     return NULL;
@@ -646,6 +782,11 @@
 
 uint16_t drizzle_column_current(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return 0;
+  }
+
   return result->column_current;
 }
 
@@ -656,6 +797,11 @@
 drizzle_return_t drizzle_column_write(drizzle_result_st *result,
                                       drizzle_column_st *column)
 {
+  if (result == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   if (drizzle_state_none(result->con))
   {
     result->column= column;
@@ -668,6 +814,11 @@
 
 void drizzle_column_set_catalog(drizzle_column_st *column, const char *catalog)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   if (catalog == NULL)
   {
     column->catalog[0]= 0;
@@ -681,6 +832,11 @@
 
 void drizzle_column_set_db(drizzle_column_st *column, const char *db)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   if (db == NULL)
   {
     column->db[0]= 0;
@@ -694,6 +850,11 @@
 
 void drizzle_column_set_table(drizzle_column_st *column, const char *table)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   if (table == NULL)
   {
     column->table[0]= 0;
@@ -708,6 +869,11 @@
 void drizzle_column_set_orig_table(drizzle_column_st *column,
                                    const char *orig_table)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   if (orig_table == NULL)
   {
     column->orig_table[0]= 0;
@@ -721,6 +887,11 @@
 
 void drizzle_column_set_name(drizzle_column_st *column, const char *name)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   if (name == NULL)
   {
     column->name[0]= 0;
@@ -747,28 +918,53 @@
 void drizzle_column_set_charset(drizzle_column_st *column,
                                 drizzle_charset_t charset)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   column->charset= charset;
 }
 
 void drizzle_column_set_size(drizzle_column_st *column, uint32_t size)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   column->size= size;
 }
 
 void drizzle_column_set_type(drizzle_column_st *column,
                              drizzle_column_type_t type)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   column->type= type;
 }
 
 void drizzle_column_set_flags(drizzle_column_st *column,
                               drizzle_column_flags_t flags)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   column->flags= flags;
 }
 
 void drizzle_column_set_decimals(drizzle_column_st *column, uint8_t decimals)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   column->decimals= decimals;
 }
 
@@ -776,8 +972,15 @@
                                       const uint8_t *default_value,
                                       size_t size)
 {
+  if (column == NULL)
+  {
+    return;
+  }
+
   if (default_value == NULL)
+  {
     column->default_value[0]= 0;
+  }
   else
   {
     if (size < DRIZZLE_MAX_DEFAULT_VALUE_SIZE)
@@ -802,6 +1005,11 @@
 
 drizzle_return_t drizzle_state_column_read(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_column_st *column;
   drizzle_column_type_drizzle_t drizzle_type;
 
@@ -905,6 +1113,11 @@
 
 drizzle_return_t drizzle_state_column_write(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   uint8_t *start= con->buffer_ptr + con->buffer_size;
   uint8_t *ptr;
   drizzle_column_st *column= con->result->column;
@@ -967,9 +1180,13 @@
   ptr+= 4;
 
   if (con->options & DRIZZLE_CON_MYSQL)
+  {
     ptr[0]= column->type;
+  }
   else
+  {
     ptr[0]= _column_type_drizzle_map_from[column->type];
+  }
   ptr++;
 
   drizzle_set_byte2(ptr, column->flags);

=== modified file 'libdrizzle/command.cc'
--- libdrizzle/command.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/command.cc	2011-12-19 16:28:27 +0000
@@ -88,6 +88,11 @@
 
 drizzle_return_t drizzle_state_command_read(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_log_debug(con->drizzle, "drizzle_state_command_read");
 
   if (con->buffer_size == 0)
@@ -135,6 +140,11 @@
   size_t free_size;
   drizzle_return_t ret;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_log_debug(con->drizzle, "drizzle_state_command_write");
 
   if (con->command_data == NULL && con->command_total != 0 &&
@@ -149,7 +159,9 @@
     start= con->buffer;
   }
   else
+  {
     start= con->buffer_ptr + con->buffer_size;
+  }
 
   if (con->command_offset == 0)
   {
@@ -175,9 +187,13 @@
     ptr= start;
     ptr[3]= 0;
     if (con->options & DRIZZLE_CON_MYSQL)
+    {
       ptr[4]= (uint8_t)(con->command);
+    }
     else
+    {
       ptr[4]= (uint8_t)(_command_drizzle_map[con->command]);
+    }
     ptr+= 5;
 
     if (con->command == DRIZZLE_COMMAND_CHANGE_USER)

=== modified file 'libdrizzle/conn.cc'
--- libdrizzle/conn.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/conn.cc	2011-12-19 16:28:27 +0000
@@ -66,12 +66,21 @@
 
 int drizzle_con_fd(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return -1;
+  }
+
   return con->fd;
 }
 
 drizzle_return_t drizzle_con_set_fd(drizzle_con_st *con, int fd)
 {
   drizzle_return_t ret;
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
 
   con->fd= fd;
 
@@ -84,8 +93,15 @@
 
 void drizzle_con_close(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   if (con->fd == -1)
+  {
     return;
+  }
 
   (void)closesocket(con->fd);
   con->fd= -1;
@@ -105,7 +121,9 @@
   drizzle_return_t ret;
 
   if ((con->events | events) == con->events)
+  {
     return DRIZZLE_RETURN_OK;
+  }
 
   con->events|= events;
 
@@ -126,6 +144,10 @@
 drizzle_return_t drizzle_con_set_revents(drizzle_con_st *con, short revents)
 {
   drizzle_return_t ret;
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
 
   if (revents != 0)
     con->options|= DRIZZLE_CON_IO_READY;
@@ -155,43 +177,82 @@
 
 drizzle_st *drizzle_con_drizzle(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
   return con->drizzle;
 }
 
 const char *drizzle_con_error(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return drizzle_error(con->drizzle);
 }
 
 int drizzle_con_errno(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return 0;
+  }
+
   return drizzle_errno(con->drizzle);
 }
 
 uint16_t drizzle_con_error_code(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return 0;
+  }
+
   return drizzle_error_code(con->drizzle);
 }
 
 const char *drizzle_con_sqlstate(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return drizzle_sqlstate(con->drizzle);
 }
 
 drizzle_con_options_t drizzle_con_options(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return drizzle_con_options_t();
+  }
+
   return drizzle_con_options_t(con->options);
 }
 
 void drizzle_con_set_options(drizzle_con_st *con,
                              drizzle_con_options_t options)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->options= options;
 }
 
 void drizzle_con_add_options(drizzle_con_st *con,
                              drizzle_con_options_t options)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->options|= options;
 
   /* If asking for the experimental Drizzle protocol, clean the MySQL flag. */
@@ -204,11 +265,21 @@
 void drizzle_con_remove_options(drizzle_con_st *con,
                                 drizzle_con_options_t options)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->options&= ~options;
 }
 
 const char *drizzle_con_host(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   if (con->socket_type == DRIZZLE_CON_SOCKET_TCP)
   {
     if (con->socket.tcp.host == NULL && !(con->options & DRIZZLE_CON_LISTEN))
@@ -222,28 +293,39 @@
 
 in_port_t drizzle_con_port(const drizzle_con_st *con)
 {
-  if (con->socket_type == DRIZZLE_CON_SOCKET_TCP)
+  if (con and con->socket_type == DRIZZLE_CON_SOCKET_TCP)
   {
     if (con->socket.tcp.port != 0)
+    {
       return con->socket.tcp.port;
+    }
 
     if (con->options & DRIZZLE_CON_MYSQL)
+    {
       return DRIZZLE_DEFAULT_TCP_PORT_MYSQL;
+    }
 
     return DRIZZLE_DEFAULT_TCP_PORT;
   }
 
-  return 0;
+  return in_port_t(0);
 }
 
 void drizzle_con_set_tcp(drizzle_con_st *con, const char *host, in_port_t port)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   drizzle_con_reset_addrinfo(con);
 
   con->socket_type= DRIZZLE_CON_SOCKET_TCP;
 
   if (host == NULL)
+  {
     con->socket.tcp.host= NULL;
+  }
   else
   {
     con->socket.tcp.host= con->socket.tcp.host_buffer;
@@ -256,19 +338,36 @@
 
 const char *drizzle_con_user(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return con->user;
 }
 
 const char *drizzle_con_password(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return con->password;
 }
 
 void drizzle_con_set_auth(drizzle_con_st *con, const char *user,
                           const char *password)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   if (user == NULL)
+  {
     con->user[0]= 0;
+  }
   else
   {
     strncpy(con->user, user, DRIZZLE_MAX_USER_SIZE);
@@ -276,7 +375,9 @@
   }
 
   if (password == NULL)
+  {
     con->password[0]= 0;
+  }
   else
   {
     strncpy(con->password, password, DRIZZLE_MAX_PASSWORD_SIZE);
@@ -286,13 +387,25 @@
 
 const char *drizzle_con_db(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return con->db;
 }
 
 void drizzle_con_set_db(drizzle_con_st *con, const char *db)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   if (db == NULL)
+  {
     con->db[0]= 0;
+  }
   else
   {
     strncpy(con->db, db, DRIZZLE_MAX_DB_SIZE);
@@ -302,76 +415,131 @@
 
 void *drizzle_con_context(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return con->context;
 }
 
 void drizzle_con_set_context(drizzle_con_st *con, void *context)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->context= context;
 }
 
 void drizzle_con_set_context_free_fn(drizzle_con_st *con,
                                      drizzle_con_context_free_fn *function)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->context_free_fn= function;
 }
 
 uint8_t drizzle_con_protocol_version(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return 0;
+  }
+
   return con->protocol_version;
 }
 
 const char *drizzle_con_server_version(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return con->server_version;
 }
 
 uint32_t drizzle_con_server_version_number(const drizzle_con_st *con)
 {
-  uint32_t major;
-  uint32_t minor;
-  uint32_t version;
-  const char *current;
-  char *end;
-
-  current= con->server_version;
-
-  major= (uint32_t)strtoul(current, &end, 10);
-  current= end + 1;
-  minor= (uint32_t)strtoul(current, &end, 10);
-  current= end + 1;
-  version= (uint32_t)strtoul(current, &end, 10);
-
-  return (major * 10000) + (minor * 100) + version;
+  if (con)
+  {
+    const char *current= con->server_version;
+    char *end;
+
+    uint32_t major= (uint32_t)strtoul(current, &end, 10);
+    current= end +1;
+    uint32_t minor= (uint32_t)strtoul(current, &end, 10);
+    current= end +1;
+    uint32_t version= (uint32_t)strtoul(current, &end, 10);
+
+    return (major * 10000) +(minor * 100) +version;
+  }
+
+  return 0;
 }
 
 uint32_t drizzle_con_thread_id(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return 0;
+  }
+
   return con->thread_id;
 }
 
 const uint8_t *drizzle_con_scramble(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return con->scramble;
 }
 
 drizzle_capabilities_t drizzle_con_capabilities(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return drizzle_capabilities_t();
+  }
+
   return drizzle_capabilities_t(con->capabilities);
 }
 
 drizzle_charset_t drizzle_con_charset(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return drizzle_charset_t();
+  }
+
   return con->charset;
 }
 
 drizzle_con_status_t drizzle_con_status(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return drizzle_con_status_t();
+  }
+
   return con->status;
 }
 
 uint32_t drizzle_con_max_packet_size(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return 0;
+  }
+
   return con->max_packet_size;
 }
 
@@ -381,8 +549,15 @@
 
 drizzle_return_t drizzle_con_connect(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   if (con->options & DRIZZLE_CON_READY)
+  {
     return DRIZZLE_RETURN_OK;
+  }
 
   if (drizzle_state_none(con))
   {
@@ -436,7 +611,13 @@
                                         drizzle_result_st *result,
                                         drizzle_return_t *ret_ptr)
 {
-  if (con->options & DRIZZLE_CON_MYSQL)
+  drizzle_return_t unused;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused;
+  }
+
+  if (con and con->options & DRIZZLE_CON_MYSQL)
   {
     return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_SHUTDOWN,
                                      "0", 1, 1, ret_ptr);
@@ -447,10 +628,9 @@
 }
 
 drizzle_result_st *drizzle_shutdown(drizzle_con_st *con,
-                                    drizzle_result_st *result, uint32_t level,
+                                    drizzle_result_st *result, uint32_t, // level is unused
                                     drizzle_return_t *ret_ptr)
 {
-  (void) level;
   return drizzle_con_shutdown(con, result, ret_ptr);
 }
 
@@ -459,6 +639,12 @@
                                 uint32_t query_id,
                                 drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused;
+  }
+
   uint32_t sent= htonl(query_id);
   return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_PROCESS_KILL,
                                    &sent, sizeof(uint32_t), sizeof(uint32_t), ret_ptr);
@@ -486,14 +672,20 @@
                                              size_t total,
                                              drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused;
+  }
+
+  if (con == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return NULL;
+  }
+
   drizzle_result_st *old_result;
 
-  drizzle_return_t unused;
-  if (ret_ptr == NULL)
-  {
-    ret_ptr= &unused;
-  }
-
   if (!(con->options & DRIZZLE_CON_READY))
   {
     if (con->options & DRIZZLE_CON_RAW_PACKET)
@@ -567,8 +759,15 @@
 
 drizzle_return_t drizzle_con_listen(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   if (con->options & DRIZZLE_CON_READY)
+  {
     return DRIZZLE_RETURN_OK;
+  }
 
   if (drizzle_state_none(con))
   {
@@ -581,25 +780,47 @@
 
 int drizzle_con_backlog(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return 0;
+  }
+
   return con->backlog;
 }
 
 void drizzle_con_set_backlog(drizzle_con_st *con, int backlog)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->backlog= backlog;
 }
 
 void drizzle_con_set_protocol_version(drizzle_con_st *con,
                                       uint8_t protocol_version)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->protocol_version= protocol_version;
 }
 
 void drizzle_con_set_server_version(drizzle_con_st *con,
                                     const char *server_version)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   if (server_version == NULL)
+  {
     con->server_version[0]= 0;
+  }
   else
   {
     strncpy(con->server_version, server_version,
@@ -610,13 +831,25 @@
 
 void drizzle_con_set_thread_id(drizzle_con_st *con, uint32_t thread_id)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->thread_id= thread_id;
 }
 
 void drizzle_con_set_scramble(drizzle_con_st *con, const uint8_t *scramble)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   if (scramble == NULL)
+  {
     con->scramble= NULL;
+  }
   else
   {
     con->scramble= con->scramble_buffer;
@@ -627,22 +860,42 @@
 void drizzle_con_set_capabilities(drizzle_con_st *con,
                                   drizzle_capabilities_t capabilities)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->capabilities= capabilities;
 }
 
 void drizzle_con_set_charset(drizzle_con_st *con, drizzle_charset_t charset)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->charset= charset;
 }
 
 void drizzle_con_set_status(drizzle_con_st *con, drizzle_con_status_t status)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->status= status;
 }
 
 void drizzle_con_set_max_packet_size(drizzle_con_st *con,
                                      uint32_t max_packet_size)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   con->max_packet_size= max_packet_size;
 }
 
@@ -666,6 +919,18 @@
                                size_t *size, size_t *total,
                                drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (con == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return NULL;
+  }
+
   if (drizzle_state_none(con))
   {
     con->packet_number= 0;
@@ -676,15 +941,31 @@
     drizzle_state_push(con, drizzle_state_packet_read);
   }
 
-  *offset= con->command_offset;
+  if (offset)
+  {
+    *offset= con->command_offset;
+  }
 
   *ret_ptr= drizzle_state_loop(con);
   if (*ret_ptr == DRIZZLE_RETURN_PAUSE)
+  {
     *ret_ptr= DRIZZLE_RETURN_OK;
-
-  *command= con->command;
-  *size= con->command_size;
-  *total= con->command_total;
+  }
+
+  if (command)
+  {
+    *command= con->command;
+  }
+
+  if (size)
+  {
+    *size= con->command_size;
+  }
+
+  if (total)
+  {
+    *total= con->command_total;
+  }
 
   return con->command_data;
 }
@@ -696,9 +977,29 @@
   size_t offset= 0;
   size_t size= 0;
 
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  size_t unused_total;
+  if (total == NULL)
+  {
+    total= &unused_total;
+  }
+
+  if (con == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return NULL;
+  }
+
   char *command_data= (char *)drizzle_con_command_read(con, command, &offset, &size, total, ret_ptr);
   if (*ret_ptr != DRIZZLE_RETURN_OK)
+  {
     return NULL;
+  }
 
   if (command_data == NULL)
   {
@@ -723,7 +1024,9 @@
   {
     command_data= (char *)drizzle_con_command_read(con, command, &offset, &size, total, ret_ptr);
     if (*ret_ptr != DRIZZLE_RETURN_OK)
+    {
       return NULL;
+    }
 
     memcpy(con->command_buffer + offset, command_data, size);
   }
@@ -741,6 +1044,11 @@
 
 void drizzle_con_reset_addrinfo(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   switch (con->socket_type)
   {
   case DRIZZLE_CON_SOCKET_TCP:
@@ -774,6 +1082,11 @@
   struct addrinfo ai;
   int ret;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_log_debug(con->drizzle, "drizzle_state_addrinfo");
 
   switch (con->socket_type)
@@ -788,11 +1101,17 @@
     }
 
     if (tcp->port != 0)
+    {
       snprintf(port, NI_MAXSERV, "%u", tcp->port);
+    }
     else if (con->options & DRIZZLE_CON_MYSQL)
+    {
       snprintf(port, NI_MAXSERV, "%u", DRIZZLE_DEFAULT_TCP_PORT_MYSQL);
+    }
     else
+    {
       snprintf(port, NI_MAXSERV, "%u", DRIZZLE_DEFAULT_TCP_PORT);
+    }
     port[NI_MAXSERV-1]= 0;
 
     memset(&ai, 0, sizeof(struct addrinfo));
@@ -842,6 +1161,11 @@
   int ret;
   drizzle_return_t dret;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_log_debug(con->drizzle, "drizzle_state_connect");
 
   if (con->fd != -1)
@@ -946,6 +1270,7 @@
   }
 
   drizzle_state_pop(con);
+
   return DRIZZLE_RETURN_OK;
 }
 
@@ -953,6 +1278,11 @@
 {
   drizzle_return_t ret;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_log_debug(con->drizzle, "drizzle_state_connecting");
 
   while (1)
@@ -980,7 +1310,9 @@
 
     ret= drizzle_con_wait(con->drizzle);
     if (ret != DRIZZLE_RETURN_OK)
+    {
       return ret;
+    }
   }
 }
 
@@ -989,6 +1321,11 @@
   drizzle_return_t ret;
   ssize_t read_size;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_log_debug(con->drizzle, "drizzle_state_read");
 
   if (con->buffer_size == 0)
@@ -1110,6 +1447,7 @@
   }
 
   drizzle_state_pop(con);
+
   return DRIZZLE_RETURN_OK;
 }
 
@@ -1118,6 +1456,11 @@
   drizzle_return_t ret;
   ssize_t write_size;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_log_debug(con->drizzle, "drizzle_state_write");
 
   while (con->buffer_size != 0)
@@ -1211,6 +1554,7 @@
   con->buffer_ptr= con->buffer;
 
   drizzle_state_pop(con);
+
   return DRIZZLE_RETURN_OK;
 }
 
@@ -1222,6 +1566,11 @@
   int opt;
   drizzle_con_st *new_con;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   for (; con->addrinfo_next != NULL;
        con->addrinfo_next= con->addrinfo_next->ai_next)
   {
@@ -1316,9 +1665,12 @@
 
   /* Report last socket() error if we couldn't find an address to bind. */
   if (con->fd == -1)
+  {
     return DRIZZLE_RETURN_ERRNO;
+  }
 
   drizzle_state_pop(con);
+
   return DRIZZLE_RETURN_OK;
 }
 
@@ -1332,6 +1684,8 @@
   struct linger linger;
   struct timeval waittime;
 
+  assert(con);
+
   ret= 1;
 
 #ifdef _WIN32

=== modified file 'libdrizzle/conn_uds.cc'
--- libdrizzle/conn_uds.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/conn_uds.cc	2011-12-19 16:28:27 +0000
@@ -44,6 +44,11 @@
 
 const char *drizzle_con_uds(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   if (con->socket_type == DRIZZLE_CON_SOCKET_UDS)
   {
     if (con->socket.uds.sockaddr.sun_path[0] != 0)
@@ -60,12 +65,19 @@
 
 void drizzle_con_set_uds(drizzle_con_st *con, const char *uds)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   drizzle_con_reset_addrinfo(con);
 
   con->socket_type= DRIZZLE_CON_SOCKET_UDS;
 
   if (uds == NULL)
+  {
     uds= "";
+  }
 
   con->socket.uds.sockaddr.sun_family= AF_UNIX;
   strncpy(con->socket.uds.sockaddr.sun_path, uds,

=== modified file 'libdrizzle/drizzle.cc'
--- libdrizzle/drizzle.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/drizzle.cc	2011-12-19 16:28:27 +0000
@@ -80,7 +80,9 @@
 const char *drizzle_verbose_name(drizzle_verbose_t verbose)
 {
   if (verbose >= DRIZZLE_VERBOSE_MAX)
+  {
     return "UNKNOWN";
+  }
 
   return _verbose_name[verbose];
 }
@@ -104,12 +106,16 @@
   {
     drizzle= new (std::nothrow) drizzle_st;
     if (drizzle == NULL)
+    {
       return NULL;
+    }
 
     drizzle->options= DRIZZLE_ALLOCATED;
   }
   else
+  {
     drizzle->options= DRIZZLE_NONE;
+  }
 
   /* @todo remove this default free flag with new API. */
   drizzle->options|= DRIZZLE_FREE_OBJECTS;
@@ -140,15 +146,15 @@
 
 drizzle_st *drizzle_clone(drizzle_st *drizzle, const drizzle_st *from)
 {
-  drizzle_con_st *con;
-
   drizzle= drizzle_create(drizzle);
   if (drizzle == NULL)
+  {
     return NULL;
+  }
 
   drizzle->options|= (from->options & int(~DRIZZLE_ALLOCATED));
 
-  for (con= from->con_list; con != NULL; con= con->next)
+  for (drizzle_con_st *con= from->con_list; con != NULL; con= con->next)
   {
     if (drizzle_con_clone(drizzle, NULL, con) == NULL)
     {
@@ -162,8 +168,15 @@
 
 void drizzle_free(drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   if (drizzle->context != NULL && drizzle->context_free_fn != NULL)
+  {
     drizzle->context_free_fn(drizzle, drizzle->context);
+  }
 
   if (drizzle->options & DRIZZLE_FREE_OBJECTS)
   {
@@ -190,83 +203,163 @@
 
 const char *drizzle_error(const drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   return (const char *)drizzle->last_error;
 }
 
 int drizzle_errno(const drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return 0;
+  }
+
   return drizzle->last_errno;
 }
 
 uint16_t drizzle_error_code(const drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return 0;
+  }
+
   return drizzle->error_code;
 }
 
 const char *drizzle_sqlstate(const drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   return drizzle->sqlstate;
 }
 
 drizzle_options_t drizzle_options(const drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return drizzle_options_t();
+  }
+
   return drizzle_options_t(drizzle->options);
 }
 
 void drizzle_set_options(drizzle_st *drizzle, drizzle_options_t options)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   drizzle->options= options;
 }
 
 void drizzle_add_options(drizzle_st *drizzle, drizzle_options_t options)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   drizzle->options|= options;
 }
 
 void drizzle_remove_options(drizzle_st *drizzle, drizzle_options_t options)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   drizzle->options&= ~options;
 }
 
 void *drizzle_context(const drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   return drizzle->context;
 }
 
 void drizzle_set_context(drizzle_st *drizzle, void *context)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   drizzle->context= context;
 }
 
 void drizzle_set_context_free_fn(drizzle_st *drizzle,
                                  drizzle_context_free_fn *function)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   drizzle->context_free_fn= function;
 }
 
 int drizzle_timeout(const drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return -1;
+  }
+
   return drizzle->timeout;
 }
 
 void drizzle_set_timeout(drizzle_st *drizzle, int timeout)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   drizzle->timeout= timeout;
 }
 
 drizzle_verbose_t drizzle_verbose(const drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return drizzle_verbose_t();
+  }
+
   return drizzle->verbose;
 }
 
 void drizzle_set_verbose(drizzle_st *drizzle, drizzle_verbose_t verbose)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   drizzle->verbose= verbose;
 }
 
 void drizzle_set_log_fn(drizzle_st *drizzle, drizzle_log_fn *function,
                         void *context)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   drizzle->log_fn= function;
   drizzle->log_context= context;
 }
@@ -275,29 +368,46 @@
                                 drizzle_event_watch_fn *function,
                                 void *context)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   drizzle->event_watch_fn= function;
   drizzle->event_watch_context= context;
 }
 
 drizzle_con_st *drizzle_con_create(drizzle_st *drizzle, drizzle_con_st *con)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   if (con == NULL)
   {
     con= new (std::nothrow) drizzle_con_st;
     if (con == NULL)
     {
       if (drizzle != NULL)
+      {
         drizzle_set_error(drizzle, "drizzle_con_create", "malloc");
+      }
+
       return NULL;
     }
 
     con->options= DRIZZLE_CON_ALLOCATED;
   }
   else
+  {
     con->options= 0;
+  }
 
   if (drizzle->con_list != NULL)
+  {
     drizzle->con_list->prev= con;
+  }
   con->next= drizzle->con_list;
   con->prev= NULL;
   drizzle->con_list= con;
@@ -354,9 +464,16 @@
 drizzle_con_st *drizzle_con_clone(drizzle_st *drizzle, drizzle_con_st *con,
                                   const drizzle_con_st *from)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   con= drizzle_con_create(drizzle, con);
   if (con == NULL)
+  {
     return NULL;
+  }
 
   /* Clear "operational" options such as IO status. */
   con->options|= (from->options & int(~(
@@ -387,25 +504,41 @@
 
 void drizzle_con_free(drizzle_con_st *con)
 {
-  if (con->context != NULL && con->context_free_fn != NULL)
+  if (con == NULL)
+  {
+    return;
+  }
+
+  if (con->context != NULL and con->context_free_fn != NULL)
+  {
     con->context_free_fn(con, con->context);
+  }
 
   if (con->drizzle->options & DRIZZLE_FREE_OBJECTS)
+  {
     drizzle_result_free_all(con);
+  }
   else if (con->drizzle->options & DRIZZLE_ASSERT_DANGLING)
+  {
     assert(con->result_list == NULL);
+  }
 
   if (con->fd != -1)
+  {
     drizzle_con_close(con);
+  }
 
   drizzle_con_reset_addrinfo(con);
 
   if (con->drizzle->con_list == con)
     con->drizzle->con_list= con->next;
+
   if (con->prev != NULL)
     con->prev->next= con->next;
+
   if (con->next != NULL)
     con->next->prev= con->prev;
+
   con->drizzle->con_count--;
 
   if (con->options & DRIZZLE_CON_ALLOCATED)
@@ -422,12 +555,12 @@
 
 drizzle_return_t drizzle_con_wait(drizzle_st *drizzle)
 {
-  drizzle_con_st *con;
+  if (drizzle == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   struct pollfd *pfds;
-  uint32_t x;
-  int ret;
-  drizzle_return_t dret;
-
   if (drizzle->pfds_size < drizzle->con_count)
   {
     pfds= (struct pollfd *)realloc(drizzle->pfds, drizzle->con_count * sizeof(struct pollfd));
@@ -441,13 +574,17 @@
     drizzle->pfds_size= drizzle->con_count;
   }
   else
+  {
     pfds= drizzle->pfds;
+  }
 
-  x= 0;
-  for (con= drizzle->con_list; con != NULL; con= con->next)
+  uint32_t x= 0;
+  for (drizzle_con_st *con= drizzle->con_list; con != NULL; con= con->next)
   {
     if (con->events == 0)
+    {
       continue;
+    }
 
     pfds[x].fd= con->fd;
     pfds[x].events= con->events;
@@ -462,6 +599,7 @@
     return DRIZZLE_RETURN_NO_ACTIVE_CONNECTIONS;
   }
 
+  int ret;
   while (1)
   {
     drizzle_log_crazy(drizzle, "poll count=%d timeout=%d", x,
@@ -474,7 +612,9 @@
     if (ret == -1)
     {
       if (errno == EINTR)
+      {
         continue;
+      }
 
       drizzle_set_error(drizzle, "drizzle_con_wait", "poll:%d", errno);
       drizzle->last_errno= errno;
@@ -491,14 +631,18 @@
   }
 
   x= 0;
-  for (con= drizzle->con_list; con != NULL; con= con->next)
+  for (drizzle_con_st *con= drizzle->con_list; con != NULL; con= con->next)
   {
     if (con->events == 0)
+    {
       continue;
+    }
 
-    dret= drizzle_con_set_revents(con, pfds[x].revents);
+    drizzle_return_t dret= drizzle_con_set_revents(con, pfds[x].revents);
     if (dret != DRIZZLE_RETURN_OK)
+    {
       return dret;
+    }
 
     x++;
   }
@@ -508,12 +652,15 @@
 
 drizzle_con_st *drizzle_con_ready(drizzle_st *drizzle)
 {
-  drizzle_con_st *con;
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
 
   /* We can't keep state between calls since connections may be removed during
      processing. If this list ever gets big, we may want something faster. */
 
-  for (con= drizzle->con_list; con != NULL; con= con->next)
+  for (drizzle_con_st *con= drizzle->con_list; con != NULL; con= con->next)
   {
     if (con->options & DRIZZLE_CON_IO_READY)
     {
@@ -527,12 +674,15 @@
 
 drizzle_con_st *drizzle_con_ready_listen(drizzle_st *drizzle)
 {
-  drizzle_con_st *con;
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
 
   /* We can't keep state between calls since connections may be removed during
      processing. If this list ever gets big, we may want something faster. */
 
-  for (con= drizzle->con_list; con != NULL; con= con->next)
+  for (drizzle_con_st *con= drizzle->con_list; con != NULL; con= con->next)
   {
     if ((con->options & (DRIZZLE_CON_IO_READY | DRIZZLE_CON_LISTEN)) ==
         (DRIZZLE_CON_IO_READY | DRIZZLE_CON_LISTEN))
@@ -555,9 +705,16 @@
                                     const char *db,
                                     drizzle_con_options_t options)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   con= drizzle_con_create(drizzle, con);
   if (con == NULL)
+  {
     return NULL;
+  }
 
   drizzle_con_set_tcp(con, host, port);
   drizzle_con_set_auth(con, user, password);
@@ -572,9 +729,16 @@
                                     const char *password, const char *db,
                                     drizzle_con_options_t options)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   con= drizzle_con_create(drizzle, con);
   if (con == NULL)
+  {
     return NULL;
+  }
 
   drizzle_con_set_uds(con, uds);
   drizzle_con_set_auth(con, user, password);
@@ -594,9 +758,16 @@
                                            int backlog,
                                            drizzle_con_options_t options)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   con= drizzle_con_create(drizzle, con);
   if (con == NULL)
+  {
     return NULL;
+  }
 
   drizzle_con_set_tcp(con, host, port);
   drizzle_con_set_backlog(con, backlog);
@@ -610,9 +781,16 @@
                                            const char *uds, int backlog,
                                            drizzle_con_options_t options)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   con= drizzle_con_create(drizzle, con);
   if (con == NULL)
+  {
     return NULL;
+  }
 
   drizzle_con_set_uds(con, uds);
   drizzle_con_set_backlog(con, backlog);
@@ -624,14 +802,24 @@
 drizzle_con_st *drizzle_con_accept(drizzle_st *drizzle, drizzle_con_st *con,
                                    drizzle_return_t *ret_ptr)
 {
-  drizzle_con_st *ready;
-  int fd;
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
 
   while (1)
   {
+    drizzle_con_st *ready;
+
     if ((ready= drizzle_con_ready_listen(drizzle)) != NULL)
     {
-      fd= accept(ready->fd, NULL, NULL);
+      int fd= accept(ready->fd, NULL, NULL);
 
       con= drizzle_con_create(drizzle, con);
       if (con == NULL)
@@ -680,6 +868,11 @@
 void drizzle_set_error(drizzle_st *drizzle, const char *function,
                        const char *format, ...)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   size_t size;
   int written;
   char log_buffer[DRIZZLE_MAX_ERROR_SIZE];
@@ -696,21 +889,39 @@
   written= vsnprintf(ptr, DRIZZLE_MAX_ERROR_SIZE - size, format, args);
   va_end(args);
 
-  if (written < 0) size= DRIZZLE_MAX_ERROR_SIZE;
-  else size+= written;
+  if (written < 0)
+  {
+    size= DRIZZLE_MAX_ERROR_SIZE;
+  }
+  else
+  {
+    size+= written;
+  }
+
   if (size >= DRIZZLE_MAX_ERROR_SIZE)
+  {
     size= DRIZZLE_MAX_ERROR_SIZE - 1;
+  }
   log_buffer[size]= 0;
 
   if (drizzle->log_fn == NULL)
+  {
     memcpy(drizzle->last_error, log_buffer, size + 1);
+  }
   else
+  {
     drizzle->log_fn(log_buffer, DRIZZLE_VERBOSE_ERROR, drizzle->log_context);
+  }
 }
 
 void drizzle_log(drizzle_st *drizzle, drizzle_verbose_t verbose,
                  const char *format, va_list args)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   char log_buffer[DRIZZLE_MAX_ERROR_SIZE];
 
   if (drizzle->log_fn == NULL)

=== modified file 'libdrizzle/field.cc'
--- libdrizzle/field.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/field.cc	2011-12-19 16:28:27 +0000
@@ -50,6 +50,18 @@
                                    size_t *size, size_t *total,
                                    drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (result == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return drizzle_field_t();
+  }
+
   if (drizzle_state_none(result->con))
   {
     if (result->field_current == result->column_count)
@@ -68,9 +80,20 @@
     *ret_ptr= DRIZZLE_RETURN_ROW_BREAK;
   }
 
-  *offset= result->field_offset;
-  *size= result->field_size;
-  *total= result->field_total;
+  if (offset)
+  {
+    *offset= result->field_offset;
+  }
+
+  if (size)
+  {
+    *size= result->field_size;
+  }
+
+  if (total)
+  {
+    *total= result->field_total;
+  }
 
   return result->field;
 }
@@ -81,6 +104,18 @@
   size_t offset= 0;
   size_t size= 0;
 
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (result == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return drizzle_field_t();
+  }
+
   drizzle_field_t field= drizzle_field_read(result, &offset, &size, total, ret_ptr);
 
   if (*ret_ptr != DRIZZLE_RETURN_OK)
@@ -141,6 +176,11 @@
                                      const drizzle_field_t field, size_t size,
                                      size_t total)
 {
+  if (result == NULL)
+  {
+    return  DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_return_t ret;
 
   if (drizzle_state_none(result->con))
@@ -180,8 +220,10 @@
 
 drizzle_return_t drizzle_state_field_read(drizzle_con_st *con)
 {
-  drizzle_return_t ret;
-
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
   drizzle_log_debug(con->drizzle, "drizzle_state_field_read");
 
   if (con->buffer_size == 0)
@@ -196,6 +238,7 @@
     con->result->field_offset= 0;
     con->result->field_size= 0;
 
+    drizzle_return_t ret;
     con->result->field_total= (size_t)drizzle_unpack_length(con, &ret);
     if (ret == DRIZZLE_RETURN_NULL_SIZE)
     {
@@ -221,9 +264,13 @@
                       con->result->field_total);
 
     if ((size_t)(con->buffer_size) >= con->result->field_total)
+    {
       con->result->field_size= con->result->field_total;
+    }
     else
+    {
       con->result->field_size= con->buffer_size;
+    }
   }
   else
   {
@@ -234,7 +281,9 @@
                                 con->result->field_offset);
     }
     else
+    {
       con->result->field_size= con->buffer_size;
+    }
   }
 
   /* This is a special case when a row is larger than the packet size. */
@@ -243,7 +292,9 @@
     con->result->field_size= con->packet_size;
 
     if (con->options & DRIZZLE_CON_RAW_PACKET)
+    {
       con->result->options|= DRIZZLE_RESULT_ROW_BREAK;
+    }
     else
     {
       drizzle_state_pop(con);
@@ -287,6 +338,11 @@
 
 drizzle_return_t drizzle_state_field_write(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   uint8_t *start= con->buffer_ptr + con->buffer_size;
   uint8_t *ptr;
   size_t free_size;
@@ -387,7 +443,9 @@
       }
 
       if (result->field_size == free_size)
+      {
         result->field= NULL;
+      }
       else
       {
         result->field+= free_size;

=== modified file 'libdrizzle/handshake.cc'
--- libdrizzle/handshake.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/handshake.cc	2011-12-19 16:28:27 +0000
@@ -104,6 +104,10 @@
   int extra_length;
   unsigned char* packet_end;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
   drizzle_log_debug(con->drizzle, "drizzle_state_handshake_server_read");
 
   /* Assume the entire handshake packet will fit in the buffer. */
@@ -231,6 +235,10 @@
 {
   uint8_t *ptr;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
   drizzle_log_debug(con->drizzle, "drizzle_state_handshake_server_write");
 
   /* Calculate max packet size. */
@@ -328,6 +336,10 @@
   size_t real_size;
   uint8_t scramble_size;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
   drizzle_log_debug(con->drizzle, "drizzle_state_handshake_client_read");
 
   /* Assume the entire handshake packet will fit in the buffer. */
@@ -477,6 +489,10 @@
   int capabilities;
   drizzle_return_t ret;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
   drizzle_log_debug(con->drizzle, "drizzle_state_handshake_client_write");
 
   /* Calculate max packet size. */
@@ -566,17 +582,21 @@
 
 drizzle_return_t drizzle_state_handshake_result_read(drizzle_con_st *con)
 {
-  drizzle_return_t ret;
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+  drizzle_log_debug(con->drizzle, "drizzle_state_handshake_result_read");
+
   drizzle_result_st result;
-
-  drizzle_log_debug(con->drizzle, "drizzle_state_handshake_result_read");
-
   if (drizzle_result_create(con, &result) == NULL)
+  {
     return DRIZZLE_RETURN_MEMORY;
+  }
 
   con->result= &result;
 
-  ret= drizzle_state_result_read(con);
+  drizzle_return_t ret= drizzle_state_result_read(con);
   if (drizzle_state_none(con))
   {
     if (ret == DRIZZLE_RETURN_OK)
@@ -588,14 +608,18 @@
         ret= DRIZZLE_RETURN_AUTH_FAILED;
       }
       else
+      {
         con->options|= DRIZZLE_CON_READY;
+      }
     }
   }
 
   drizzle_result_free(&result);
 
   if (ret == DRIZZLE_RETURN_ERROR_CODE)
+  {
     return DRIZZLE_RETURN_HANDSHAKE_FAILED;
+  }
 
   return ret;
 }

=== modified file 'libdrizzle/pack.cc'
--- libdrizzle/pack.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/pack.cc	2011-12-19 16:28:27 +0000
@@ -101,6 +101,18 @@
   uint64_t length;
   uint8_t bytes;
 
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (con == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return 0;
+  }
+
   if (con->buffer_ptr[0] < 251)
   {
     length= (uint64_t)(con->buffer_ptr[0]);
@@ -146,6 +158,11 @@
 
 uint8_t *drizzle_pack_string(char *string, uint8_t *ptr)
 {
+  if (string == NULL)
+  {
+    return NULL;
+  }
+
   uint64_t size= strlen(string);
 
   ptr= drizzle_pack_length(size, ptr);
@@ -162,9 +179,13 @@
                                        uint64_t max_length)
 {
   drizzle_return_t ret= DRIZZLE_RETURN_OK;
-  uint64_t length;
-
-  length= drizzle_unpack_length(con, &ret);
+
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
+  uint64_t length= drizzle_unpack_length(con, &ret);
   if (ret != DRIZZLE_RETURN_OK)
   {
     if (ret == DRIZZLE_RETURN_NULL_SIZE)
@@ -199,6 +220,18 @@
 uint8_t *drizzle_pack_auth(drizzle_con_st *con, uint8_t *ptr,
                            drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (con == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return NULL;
+  }
+
   if (con->user[0] != 0)
   {
     memcpy(ptr, con->user, strlen(con->user));
@@ -267,7 +300,6 @@
 static drizzle_return_t _pack_scramble_hash(drizzle_con_st *con,
                                             uint8_t *buffer)
 {
-  uint32_t x;
   SHA1_CTX ctx;
   uint8_t hash_tmp1[SHA1_DIGEST_LENGTH];
   uint8_t hash_tmp2[SHA1_DIGEST_LENGTH];
@@ -304,9 +336,11 @@
   SHA1Final(buffer, &ctx);
 
   /* Fourth, xor the last hash against the first password hash. */
-  x= 0;
-  for (; x < SHA1_DIGEST_LENGTH; x++)
+  uint32_t x;
+  for (uint32_t x= 0; x < SHA1_DIGEST_LENGTH; x++)
+  {
     buffer[x]= buffer[x] ^ hash_tmp1[x];
+  }
 
   return DRIZZLE_RETURN_OK;
 }

=== modified file 'libdrizzle/query.cc'
--- libdrizzle/query.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/query.cc	2011-12-19 16:28:27 +0000
@@ -55,9 +55,12 @@
                                      const char *query, 
                                      drizzle_return_t *ret_ptr)
 {
-  size_t size;
+  if (query == NULL)
+  {
+    return NULL;
+  }
 
-  size= strlen(query);
+  size_t size= strlen(query);
 
   return drizzle_con_command_write(con, result, DRIZZLE_COMMAND_QUERY,
                                    (uint8_t *)query, size, size, ret_ptr);
@@ -80,9 +83,16 @@
                                     drizzle_query_options_t options,
                                     void *context)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   query= drizzle_query_create(drizzle, query);
   if (query == NULL)
+  {
     return NULL;
+  }
 
   drizzle_query_set_con(query, con);
   drizzle_query_set_result(query, result);
@@ -96,6 +106,11 @@
 drizzle_query_st *drizzle_query_create(drizzle_st *drizzle,
                                        drizzle_query_st *query)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
   if (query == NULL)
   {
     query= new (std::nothrow) drizzle_query_st;
@@ -109,7 +124,9 @@
     query->options|= DRIZZLE_CON_ALLOCATED;
   }
   else
+  {
     memset(query, 0, sizeof(drizzle_query_st));
+  }
 
   query->drizzle= drizzle;
 
@@ -125,13 +142,20 @@
 
 void drizzle_query_free(drizzle_query_st *query)
 {
+  if (query == NULL)
+  {
+    return;
+  }
+
   if (query->context != NULL && query->context_free_fn != NULL)
     query->context_free_fn(query, query->context);
 
   if (query->drizzle->query_list == query)
     query->drizzle->query_list= query->next;
+
   if (query->prev)
     query->prev->next= query->next;
+
   if (query->next)
     query->next->prev= query->prev;
   query->drizzle->query_count--;
@@ -144,6 +168,11 @@
 
 void drizzle_query_free_all(drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   while (drizzle->query_list != NULL)
   {
     drizzle_query_free(drizzle->query_list);
@@ -152,80 +181,161 @@
 
 drizzle_con_st *drizzle_query_con(drizzle_query_st *query)
 {
+  if (query == NULL)
+  {
+    return NULL;
+  }
+
   return query->con;
 }
 
 void drizzle_query_set_con(drizzle_query_st *query, drizzle_con_st *con)
 {
+  if (query == NULL)
+  {
+    return;
+  }
+
   query->con= con;
 }
 
 drizzle_result_st *drizzle_query_result(drizzle_query_st *query)
 {
+  if (query == NULL)
+  {
+    return NULL;
+  }
+
   return query->result;
 }
 
 void drizzle_query_set_result(drizzle_query_st *query,
                               drizzle_result_st *result)
 {
+  if (query == NULL)
+  {
+    return;
+  }
+
   query->result= result;
 }
 
 char *drizzle_query_string(drizzle_query_st *query, size_t *size)
 {
-  *size= query->size;
+  if (query == NULL)
+  {
+    return NULL;
+  }
+
+  if (size)
+  {
+    *size= query->size;
+  }
+
   return (char *)(query->string);
 }
 
 void drizzle_query_set_string(drizzle_query_st *query, const char *string,
                               size_t size)
 {
+  if (query == NULL)
+  {
+    return;
+  }
+
   query->string= string;
   query->size= size;
 }
 
 drizzle_query_options_t drizzle_query_options(drizzle_query_st *query)
 {
+  if (query == NULL)
+  {
+    return drizzle_query_options_t();
+  }
+
   return drizzle_query_options_t(query->options);
 }
 
 void drizzle_query_set_options(drizzle_query_st *query,
                                drizzle_query_options_t options)
 {
+  if (query == NULL)
+  {
+    return;
+  }
+
   query->options= options;
 }
 
 void drizzle_query_add_options(drizzle_query_st *query,
                                drizzle_query_options_t options)
 {
+  if (query == NULL)
+  {
+    return;
+  }
+
   query->options|= options;
 }
 
 void drizzle_query_remove_options(drizzle_query_st *query,
                                   drizzle_query_options_t options)
 {
+  if (query == NULL)
+  {
+    return;
+  }
+
   query->options&= ~options;
 }
 
 void *drizzle_query_context(drizzle_query_st *query)
 {
+  if (query == NULL)
+  {
+    return NULL;
+  }
+
   return query->context;
 }
 
 void drizzle_query_set_context(drizzle_query_st *query, void *context)
 {
+  if (query == NULL)
+  {
+    return;
+  }
+
   query->context= context;
 }
 
 void drizzle_query_set_context_free_fn(drizzle_query_st *query,
                                        drizzle_query_context_free_fn *function)
 {
+  if (query == NULL)
+  {
+    return;
+  }
+
   query->context_free_fn= function;
 }
 
 static void drizzle_query_run_state(drizzle_query_st* query,
                                     drizzle_return_t* ret_ptr)
 {
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (query == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return;
+  }
+
   switch (query->state)
   {
   case DRIZZLE_QUERY_STATE_INIT:
@@ -264,6 +374,18 @@
 drizzle_query_st *drizzle_query_run(drizzle_st *drizzle,
                                     drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (drizzle == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return NULL;
+  }
+
   int options;
   drizzle_query_st *query;
   drizzle_con_st *con;
@@ -341,10 +463,15 @@
 
 drizzle_return_t drizzle_query_run_all(drizzle_st *drizzle)
 {
-  drizzle_return_t ret;
+  if (drizzle == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
 
   while (drizzle->query_new > 0 || drizzle->query_running > 0)
   {
+    drizzle_return_t ret;
+
     (void)drizzle_query_run(drizzle, &ret);
     if (ret != DRIZZLE_RETURN_OK && ret != DRIZZLE_RETURN_ERROR_CODE)
       return ret;
@@ -355,6 +482,11 @@
 
 ssize_t drizzle_safe_escape_string(char *to, size_t max_to_size, const char *from, size_t from_size)
 {
+  if (to == NULL or max_to_size == 0 or from == NULL or from_size == 0)
+  {
+    return -1;
+  }
+
   ssize_t to_size= 0;
   char newchar;
   const char *end;
@@ -423,6 +555,11 @@
 
 size_t drizzle_hex_string(char *to, const char *from, size_t from_size)
 {
+  if (to == NULL or from == NULL or from_size == 0)
+  {
+    return size_t(-1);
+  }
+
   static const char hex_map[]= "0123456789ABCDEF";
   const char *from_end;
 

=== modified file 'libdrizzle/row.cc'
--- libdrizzle/row.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/row.cc	2011-12-19 16:28:27 +0000
@@ -48,6 +48,18 @@
 
 uint64_t drizzle_row_read(drizzle_result_st *result, drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (result == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return 0;
+  }
+
   if ((result->column_current != result->column_count) && (!(result->options & DRIZZLE_RESULT_BUFFER_COLUMN)))
   {
     drizzle_set_error(result->con->drizzle, "drizzle_row_read", "cannot retrieve rows until all columns are retrieved");
@@ -69,6 +81,18 @@
 drizzle_row_t drizzle_row_buffer(drizzle_result_st *result,
                                  drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused_ret;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused_ret;
+  }
+
+  if (result == NULL)
+  {
+    *ret_ptr= DRIZZLE_RETURN_INVALID_ARGUMENT;
+    return 0;
+  }
+
   size_t total;
   drizzle_field_t field;
   drizzle_row_t row;
@@ -122,6 +146,11 @@
 
 void drizzle_row_free(drizzle_result_st *result, drizzle_row_t row)
 {
+  if (result == NULL)
+  {
+    return;
+  }
+
   for (uint16_t x= 0; x < result->column_count; x++)
   {
     drizzle_field_free(row[x]);
@@ -132,13 +161,25 @@
 
 size_t *drizzle_row_field_sizes(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return NULL;
+  }
+
   return result->field_sizes;
 }
 
 drizzle_row_t drizzle_row_next(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return drizzle_row_t();
+  }
+
   if (result->row_current == result->row_count)
+  {
     return NULL;
+  }
 
   result->field_sizes= result->field_sizes_list[result->row_current];
   result->row_current++;
@@ -147,6 +188,11 @@
 
 drizzle_row_t drizzle_row_prev(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return drizzle_row_t();
+  }
+
   if (result->row_current == 0)
     return NULL;
 
@@ -157,12 +203,22 @@
 
 void drizzle_row_seek(drizzle_result_st *result, uint64_t row)
 {
+  if (result == NULL)
+  {
+    return;
+  }
+
   if (row <= result->row_count)
     result->row_current= row;
 }
 
 drizzle_row_t drizzle_row_index(drizzle_result_st *result, uint64_t row)
 {
+  if (result == NULL)
+  {
+    return drizzle_row_t();
+  }
+
   if (row >= result->row_count)
     return NULL;
 
@@ -171,6 +227,11 @@
 
 uint64_t drizzle_row_current(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return 0;
+  }
+
   return result->row_current;
 }
 
@@ -180,8 +241,15 @@
 
 drizzle_return_t drizzle_row_write(drizzle_result_st *result)
 {
+  if (result == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   if (drizzle_state_none(result->con))
+  {
     drizzle_state_push(result->con, drizzle_state_row_write);
+  }
 
   return drizzle_state_loop(result->con);
 }
@@ -192,6 +260,11 @@
 
 drizzle_return_t drizzle_state_row_read(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_log_debug(con->drizzle, "drizzle_state_row_read");
 
   if (con->packet_size != 0 && con->buffer_size < con->packet_size && 
@@ -233,6 +306,11 @@
 
 drizzle_return_t drizzle_state_row_write(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   uint8_t *start= con->buffer_ptr + con->buffer_size;
 
   drizzle_log_debug(con->drizzle, "drizzle_state_row_write");

=== modified file 'libdrizzle/state.cc'
--- libdrizzle/state.cc	2011-12-16 06:32:39 +0000
+++ libdrizzle/state.cc	2011-12-19 16:28:27 +0000
@@ -44,6 +44,11 @@
 
 drizzle_return_t drizzle_state_loop(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   while (drizzle_state_none(con) == false)
   {
     drizzle_return_t ret= con->state_stack[con->state_current - 1](con);
@@ -64,6 +69,11 @@
 
 drizzle_return_t drizzle_state_packet_read(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
   drizzle_log_debug(con->drizzle, "drizzle_state_packet_read");
 
   if (con->buffer_size < 4)

